@@ -12,7 +12,7 @@ use JSON;
 use LWP::UserAgent;
 use List::Util qw(none);
 
-our $VERSION = '1.02';
+our $VERSION = '2.0.2';
 
 use parent qw(Class::Accessor::Fast);
 
@@ -20,6 +20,7 @@ __PACKAGE__->follow_best_practice;
 
 __PACKAGE__->mk_accessors(
   qw(
+    body
     css
     engine
     git_email
@@ -30,7 +31,6 @@ __PACKAGE__->mk_accessors(
     mode
     no_title
     raw
-    body
     render
     title
   )
@@ -38,12 +38,14 @@ __PACKAGE__->mk_accessors(
 
 use Readonly;
 
-Readonly our $GITHUB_API  => 'https://api.github.com/markdown';
-Readonly our $EMPTY       => q{};
-Readonly our $SPACE       => q{ };
-Readonly our $TOC_TITLE   => 'Table of Contents';
-Readonly our $TOC_BACK    => 'Back to Table of Contents';
-Readonly our $DEFAULT_CSS => 'https://cdn.simplecss.org/simple-v1.css';
+Readonly::Scalar our $GITHUB_API  => 'https://api.github.com/markdown';
+Readonly::Scalar our $EMPTY       => q{};
+Readonly::Scalar our $SPACE       => q{ };
+Readonly::Scalar our $TOC_TITLE   => 'Table of Contents';
+Readonly::Scalar our $TOC_BACK    => 'Back to Table of Contents';
+Readonly::Scalar our $DEFAULT_CSS => 'https://cdn.simplecss.org/simple-v1.css';
+Readonly::Scalar our $TRUE        => 1;
+Readonly::Scalar our $FALSE       => 0;
 
 caller or __PACKAGE__->main;
 
@@ -55,7 +57,7 @@ sub new {
   my %options = ref $args[0] ? %{ $args[0] } : @args;
 
   $options{title}  //= $TOC_TITLE;
-  $options{css}    //= $DEFAULT_CSS;
+  $options{css}    //= $options{nocss} ? $EMPTY : $DEFAULT_CSS;
   $options{mode}   //= 'markdown';
   $options{engine} //= 'github';
 
@@ -181,22 +183,21 @@ sub _render_with_text_markdown {
 ########################################################################
   my ($self) = @_;
 
-  eval { require Text::Markdown; };
+  eval { require Text::Markdown::Discount; };
 
   if ($EVAL_ERROR) {
-    croak "no Text::Markdown available...try using GitHub API.\n$EVAL_ERROR";
+    carp "no Text::Markdown::Discount available...using GitHub API.\n$EVAL_ERROR";
+    return $self->_render_with_github;
   }
 
-  my $tm = Text::Markdown->new( trust_list_start_value => 1 );
-
   my $markdown = $self->get_markdown;
-  my $html     = Text::Markdown::markdown($markdown);
+  my $html     = Text::Markdown::Discount::markdown($markdown);
 
   if ( $self->get_raw ) {
     $self->set_html($html);
   }
   else {
-    $self->fix_anchors( Text::Markdown::markdown($markdown) );
+    $self->fix_anchors( Text::Markdown::Discount::markdown($markdown) );
   }
 
   return $self;
@@ -211,10 +212,14 @@ sub fix_anchors {
 
   my $html;
 
+  # sample:
+  # <div class="markdown-heading"><h1 class="heading-element">Table of Contents</h1><a id="table-of-contents" class="anchor" aria-label="Permalink: Table of Contents" href="#table-of-contents"><span aria-hidden="true" class="octicon octicon-link"></span></a></div>
+
   while ( my $line = <$fh> ) {
     # remove garbage if there...
-    if ( $line =~ /^\s*<h(\d+)><a (.*)>(.*)<\/a>(.*)/xsm ) {
-      $line = "<h$1>$3$4</h$1>\n";
+
+    if ( $line =~ m{<h(\d+)([^>]*?)>([^<]+?)</h\d+><a(.*?)/a>(.*?)}xsm ) {
+      $line = "<a$4$5/a><h$1$2>$3</h$1>\n";
     }
 
     $html .= _fix_header($line);
@@ -238,13 +243,12 @@ sub _fix_header {
 
   $anchor = lc $anchor;
   $anchor =~ s/\s+/-/gxsm;                        # spaces become '-'
-  $anchor =~ s/[\@'(),\`]//xsmg;                  # known weird characters, but expect more
+  $anchor =~ s/[.:\?_.\@'(),\`\*]//xsmg;          # known weird characters, but expect more
   $anchor =~ s/\///xsmg;
 
   $line
-    = sprintf qq{<h%d><a id="%s" class="anchor" href="#%s"></a>%s</h%d>\n},
-    $hn,
-    $anchor, $anchor, $header, $hn;
+    = sprintf qq{<a id="%s" class="anchor" href="#%s"></a><h%s>%s</h%d>\n},
+    $anchor, $anchor, $hn, $header, $hn;
 
   return $line;
 }
@@ -303,7 +307,8 @@ sub _render_with_github {
     return $self->fix_anchors( $rsp->content );
   }
   else {
-    return $self->fix_github_html( $rsp->content );
+    $self->fix_anchors( $rsp->content );
+    return $self->fix_github_html( $self->get_html );
   }
 }
 
@@ -357,9 +362,18 @@ sub create_toc {
 
   my $toc = $self->get_no_title ? $EMPTY : "# \@TOC_TITLE\@\n\n";
 
+  my $in_code_block = $FALSE;
+
   while (<$fh>) {
     chomp;
 
+    if (/^\s*```/xsm) {
+      $in_code_block = $in_code_block ? $FALSE : $TRUE;
+      next;
+    }
+
+    next if $in_code_block;
+
     /^(\#+)\s+(.*?)$/xsm && do {
       my $level = $1;
 
@@ -373,7 +387,7 @@ sub create_toc {
 
       $link =~ s/\s+/-/gxsm;  # spaces become '-'
 
-      $link =~ s/[\@'(),\`]//xsmg;  # known weird characters, but expect more
+      $link =~ s/[\?\_:.\@'(),\`\*]//xsmg;  # known weird characters, but expect more
 
       $link =~ s/\///xsmg;
 
@@ -478,12 +492,35 @@ Markdown::Render - Render markdown as HTML
 
  $md->render_markdown->print_html;
 
+...or from the command line to create HTML
+
+ md-utils.pl -r README.md > README.html
+
+...or from the command line to replace render custom tags
+
+ md-utils.pl README.md.in > README.md
+
 =head1 DESCRIPTION
 
-Renders markdown as HTML using either GitHub's API or L<Text::Markdown>. Optionally adds
-additional metadata to markdown document using custom tags.
+Renders markdown as HTML using either GitHub's API or
+L<Text::Markdown::Discount>. Optionally adds additional metadata to markdown
+document using custom tags.
 
-See L<README.md|https://github.com/rlauer6/markdown-utils/blob/master/README.md> for more details.
+See
+L<README.md|https://github.com/rlauer6/markdown-utils/blob/master/README.md>
+for more details.
+
+I<Note: This module originally used L<Text::Markdown> as an
+alternative to using the GitHub API however, there are too many bugs
+and idiosyncracies in that module. This module will now use
+L<Text::Markdown::Discount> which is not only faster, but seems to be
+more compliant with GFM.>
+
+I<Note: Text::Markdown::Discount relies on the text-markdown library
+which did not actually support all of the markdown features (including
+code fencing).  You can find an updated version of
+L<Text::Markdown::Discount> here:
+L<https://github.com/rlauer6/text-markdown-discount>>
 
 =head1 METHODS AND SUBROUTINES
 
@@ -512,7 +549,7 @@ Name of the git user that is used in the C<GIT_USER> tag.
 
 =item git_email
 
-Email address of the git user is used in the C<GIT_EMAIL> tag.
+Email address of the git user that is used in the C<GIT_EMAIL> tag.
 
 =item infile
 
@@ -530,7 +567,8 @@ default: markdown
 
 =item no_title
 
-Boolean that indicates that no title should be added to the table of contents.
+Boolean that indicates that no title should be added to the table of
+contents.
 
 default: false
 
@@ -572,7 +610,7 @@ Outputs the fully rendered HTML page.
 URL of a CSS style sheet to include in the head section. If no CSS
 file option is passed a default CSS file will b used. If a CSS element
 is passed but it is undefined or empty, then no CSS will be specified
-i the final document.
+in the final document.
 
 =item title
 
@@ -585,10 +623,15 @@ will be added to the document.
 
 =head1 AUTHOR
 
-Rob Lauer - rclauer@gmail.com
+Rob Lauer - rlauer6@comcast.net
+
+=head1 LICENSE
+
+This software is licensed under the same terms as Perl.
 
 =head1 SEE OTHER
 
 L<GitHub Markdown API|https://docs.github.com/en/rest/markdown>
+L<Text::Markdown::Discount>
 
 =cut
